use warnings;
use strict;
use Array::Diff;
use PPI::Document;
use Data::Dumper;
use List::Util qw(uniq);

my $source = <<'EOS';
use ModuleA;

ModuleA::func1;
Module::A::func2;
ModuleA::func3();
Module::A::func4();
ModuleB->method1;
Module::B->method2;
ModuleB->method3();
Module::B->method4();
EOS

my $doc         = PPI::Document->new( \$source );
my $incs        = $doc->find('PPI::Statement::Include');
my $inc_modules = [ map { $_->module } @$incs ];

my $words = $doc->find('PPI::Token::Word');

my $maybe_need_modules = [];

for my $word (@$words) {
    my $prev = $word->sprevious_sibling;
    my $next = $word->snext_sibling;

    # パターン2の抽出
    if ( $word->method_call && $prev eq '->' ) {
        my $module = $prev->sprevious_sibling;
        my $func   = $word;
        if ( $module && $func ) {
            push @$maybe_need_modules, $module->content;
        }
    }
    elsif ( $next ne '->' ) {

        # パターン1の抽出
        my ( $module, $func ) = $word =~ /((?:[A-Z]\w*::)+)(\w+)/;
        if ( $module && $func ) {
            $module = substr( $module, 0, -2 );
            push @$maybe_need_modules, $module;
        }
    }
}

my $need_modules =
  Array::Diff->diff( [ uniq @$inc_modules ], [ uniq @$maybe_need_modules ] )
  ->added;

# use されていないモジュールが抽出される
print "need modules\n";
print Dumper $need_modules;
