use lib qw(lib);
use warnings;
use strict;
use File::Find qw(find);
use Data::Dumper;
use Module::Load;

# ライブラリのインクルードパス
use constant { LIB_FILE_PATHS => [qw(lib)], };

sub get_lib_files {
    my $files = [];

    my $process = sub {
        my $file = $_;

        if ( $file =~ /\.pm/ ) {
            push @$files, $file;
        }
    };

    for my $path ( LIB_FILE_PATHS->@* ) {
        find(
            {
                wanted   => \&{$process},
                no_chdir => 1,
            },
            $path
        );
    }

    return $files;
}

my $module_to_funcs = {};
my $lib_filenames   = get_lib_files();

# @lib_filenames は @INC のパス以下のファイル
for my $filename (@$lib_filenames) {

    # lib/ModuleA.pm を ModuleA に変換
    my $module = $filename;
    $module =~ s/lib\///;
    $module =~ s/\//::/g;
    $module =~ s/\.pm$//;

    # 一時的にシンボリックリファレンスを許可する
    no strict qw(refs);

    # Module::Loadモジュールのloadメソッド
    load $module;

    $module_to_funcs->{$module} //= [];
    push $module_to_funcs->{$module}->@*, @{ $module . '::EXPORT' };
}

my $func_to_module = {
    map {
        my $module = $_;
        map { $_ => $module } $module_to_funcs->{$module}->@*
    } keys %$module_to_funcs
};

print "func to module\n";
print Dumper $func_to_module;

my $candidates = [
    qw(
      funcA
      funcB2
    )
];

print "candidates\n";
print Dumper $candidates;

my $maybe_need_modules = [];

for my $func (@$candidates) {

    # $func_to_module は $module_to_funcs から作れる
    my $module = $func_to_module->{$func};
    push @$maybe_need_modules, $module if $module;
}

print "maybe need modules\n";
print Dumper $maybe_need_modules;
